/****
  
 See READ_ME.md for compile instructions

 ****/


#define DEBUG  0       // if 0 then debug does not display

#if DEBUG == 1
#define debug(...)    Serial.printf(__VA_ARGS__)
#else
#define debug(...) 
#endif

#define SERIALON 0     // if 1 then USB serial  port is active

#include "hardware/PWM.h"  // this and "pwm.h" both work
#include <Wire.h>
#include "GPSSetup.h"
#include "setDivider.h"
#include "OLED.h"
#include "lookForSats.h"
#include "getGPSData.h"
#include "VcVoltage.h"
#include "picoTemp.h"
#include <RotaryEncoder.h>  // by Matthias Hertel
#include <EEPROM.h>         // this must be included before EEPROMFrequency.h
#include "EEPROMFrequency.h"
#include "freqToString.h"
#include "setLED.h"
#include "runSi5351.h"      // my version
#include "show.h"
#include "pressed.h"
#include "freqSetup.h"
#include "freqLoop.h"
#include "underScore.h"

// Pico Zero pin allocations for the divider
const int outPin = 14; 
const int inPin  = 15; 

#define button 8

String Time = "";
int counter = 0;
bool freqMode = false;
long unsigned frequency = 0;
bool resetUnderScore = false;
long epromFrequency;
long gotIt = 0;
bool timeLow = false;

void setup() {

  // Initialize NeoPixel
  setLED::begin();
  setLED::clear();
  setLED::set("white");

  // set up switch and rotary
  pinMode(button, INPUT_PULLUP);
  pinMode(A, INPUT_PULLUP);
  pinMode(B, INPUT_PULLUP);

   // ADC to use 12 bits
   analogReadResolution(12); 

   // set power supply to PWM mode for lower noise
    pinMode(23, OUTPUT);
    digitalWrite(23, HIGH);

   // PWM setup  
   setDivider::set(outPin, inPin);

   // set I2C to run at 400 kHz
   Wire.setClock(400000L);
   Wire.begin();  

   // Set Si5351 reference frequency
   si5351.begin(10000000UL);  // for 25 MHz use  25000000UL
  
#if SERIALON == 1
    // set the USB serial port to 115200 baud
    Serial.begin(115200);
    while(!Serial)  // wait for serial port to be available.
      delay(1);
#endif
    
    // clear the LED
    setLED::clear();

    debug("Neo-7+ GPSDO \n V1.0 November 2024 \n setting Serial1 \n");
 
    debug("setting Serial1");

    setLED::set("blue");

    // set GPS port to 9600
    Serial1.begin(9600);
    // set the timeout for reading data from the GPS via Serial1
    Serial1.setTimeout(1500); // set to 1.5 seconds

    // delay for GPS to start 
    delay(2000);

    debug("Serial1 set"); 

    // Check that the OLED is working
    if (!OLED::check()){
      debug("SSD1306 allocation failed");
      // flash red LED and halt, can't show a graphic!
      show::error(100, false);
    }
  
    debug("OLED working"); 

    // set up EEPROM to store 4 bytes
    EEPROM.begin(4);

    // Its working so show the globe image
    OLED::start();

    // setup GPS, show status on bar graph
    // or Error screen and flash red LED if can't access GPS 
    if (!GPSSetup::Setup()){
      show::error(500, true);
    }

    // all done so show thumbs up!
    OLED::thumb();
    delay (1000);

    // Show main screen, 'UTC', 'v', '0C' and 'Sats' 
    OLED::main();
    // get frequency from the EEPROM and show it but will only be shown if number of satellites > 2
    frequency = EEPROMFrequency::read();
    OLED::show(frequency, false);
    // set Si5351 frequency from the EEPROM if not 0
    if (frequency != 0)
      si5351.setOutput(frequency);
    // all working so show green Pixels
    setLED::set("green"); 

}

void loop() {
  // look if the button is being held low, must be > 1 second to move to frequency mode.
     int state = pressed::low(); 
        if (state != 2)        // state = 0 if not pressed, 1 if < 1 second and 2 if > 1 second
           freqMode = false;   // not held low for > 1 second so nothing more to do
        //button was pressed for > 1 second
        else {
          freqSetup::now(frequency);
          freqMode = true;
        }
      // stay in this loop to use the rotary & set underscore position using button press < 1 second. Press button > 1 second  to exit.
      while (freqMode) {
         // change the frequency if the rotary is moved
         freqLoop::now(frequency);

        // check if the button is pressed again for how long
        state = pressed::low(); 
        if (state == 1)                   // short press so apply to underscore
            underScore::set();
        else if (state == 2) {  // button was pressed for > 1 second so move back to main screen
            debug("Button was pressed > 1 second so moving to main mode\n");
            // If frequency is < 8kHz  or != 0 then show error meaasge for 3 seconds and don't save.
            if (frequency < 8000 & frequency != 0 ) {
                // show red LED
                setLED::set("red");
                OLED::wrong(false);  // displays < 8 kHz frequency for 3 seconds then stays in this loop.
                setLED::set("green");
            }
            // frequency is OK so exit the loop and set up all required to go back to main
            else {
              // save the frequency in the EEPROM
              EEPROMFrequency::save(frequency);
              // display "Frequency Saved" on OLED for a second if not 0 frequency
              if (frequency != 0)
                OLED::freqSaved();
              // Show main screen, 'UTC', 'v', '0C'  and 'Sats'
              OLED::main();
              // show frequency on the main screen if not 0.
              OLED::show(frequency, false);
              freqMode = false;  // this exits the loop
            }  // end else
          }  // end of state == 2
      } // end while frequency mode
 
  // Read GPS Serial input for number of satellites and current time 
  // if not available for > 3 secomds then show error
  if (Serial1.available()) {
    gotIt = millis();
    getGPSData::sats(satNumber, Time);
    // if number of satellites is too low then show finding satellites screen 
    if (satNumber < 3){
      lookForSats::search();          // will stay here if insufficient found
      OLED::main();                   // have enough satellites now so show main screen
      OLED::show(frequency, false);   // and show the frequency now
    }
    // update the time 
    OLED::time(Time);  
    // show number of satellites
    OLED::numberSats(satNumber); 
    // show Vc & if PLL locked or not
    OLED::volts(VcVoltage::volts());
    // show Pico temperature
    OLED::temp(picoTemp::get());
    }
    // data from GPS not received so show error message and flash red LED.
    else {
      if (millis() - gotIt > 3000){
        show::error(200, true);
      }
    }
}






